#!/usr/bin/env python

#
# Author: Pawel Stolowski <pawel.stolowski@ubuntu.com>
#
# Released under the terms of GNU General Public License v3
#

from gi.repository import Unity
from gi.repository import GLib, Gio, GObject
from subprocess import Popen, PIPE
import os
import re

DBUS_NAME = 'com.canonical.Unity.Lens.manpages'
DBUS_PATH = '/com/canonical/unity/lens/manpages'
SCOPE_DBUS_PATH = 'com/canonical/unity/scope/manpages'
UNKNOWN_MAN_CATEGORY = 9

class ManPagesScope(Unity.Scope):
	def __init__(self):
		super(ManPagesScope, self).__init__(dbus_path=SCOPE_DBUS_PATH)
		self.connect('notify::active-search', self._on_search_changed)
		self.connect('notify::active-global-search', self._on_global_search_changed)
		self.connect('filters-changed', self._on_search_changed)
		self.connect('activate-uri', self.activate_uri)
		self.icon = Gio.ThemedIcon.new('text-x-generic').to_string()
        #
        # regex_apropos for capturing 'apropos' results: manpage name (section)  - short description
		self.regex_apropos = re.compile('^(.+?)\s+\((\d+)\)\s+-\s(.+?)$')
		self.regex_section2cat = re.compile('^.*?(\d+)')

	def do_apropos(self, search_string):
		"""
		   	Run 'apropos <search string>' and parse, then return its results
		"""
		result = []
		if len(search_string) > 2:   # don't run apropos for strings shorter than 3 chars
			apropos = Popen(["apropos", search_string], stdout=PIPE)
			os.waitpid(apropos.pid, 0)
			out = apropos.communicate()[0]
			for line in out.split("\n"):
				m = self.regex_apropos.match(line)
				if m:
					result.append((m.group(1), m.group(2), m.group(3)))
		return result

	def activate_uri(self, scope, uri):
		return Unity.ActivationResponse.new(Unity.HandledType.HIDE_DASH, "'uri'")

	def _on_search_changed (self, scope, param_spec):
		res = self.do_apropos(self.props.active_search.props.search_string)
		self._update_model(scope.props.results_model, res)
		self.props.active_search.emit('finished')

	def _on_global_search_changed (self, scope, param_spec):
		res = self.do_apropos(self.props.active_global_search.props.search_string)
		self._update_model(scope.props.global_results_model, res)
		self.props.active_global_search.emit('finished')

	def build_path(self, name, section):
		return name + '(' + section + ')'

	def man_section_to_category(self, section):
		m = self.regex_section2cat.match(section)
		if m:
			return int(m.group(1)) - 1
		return UNKNOWN_MAN_CATEGORY

	def _update_model(self, model, result):
		model.clear()
		for res in result:
			uri = 'man:' + self.build_path(res[0], res[1])
			model.append(
                    uri,
					self.icon,
					self.man_section_to_category(res[1]),
					'x-scheme-handler/man',
					res[0] + ' (' + res[1] + ')',
					res[2],
				    uri	
					)

if __name__ == '__main__':
	session_bus_connection = Gio.bus_get_sync(Gio.BusType.SESSION, None)
	session_bus = Gio.DBusProxy.new_sync(session_bus_connection, 0, None, 'org.freedesktop.DBus', '/org/freedesktop/dbus', 'org.freedesktop.DBus', None,)
	res =session_bus.RequestName('(su)', DBUS_NAME, 0x4)

	lens = Unity.Lens.new(DBUS_PATH, 'manpages')
	lens.props.search_hint = "Search man pages"
	lens.props.visible = True
	lens.props.search_in_global = True
	icon = Gio.ThemedIcon.new("text-x-generic")
	lens.props.categories = [
				Unity.Category.new("General commands", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("System calls", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Library functions", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Special files", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("File formats and conventions", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Games and screensavers", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Miscellanea", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("System administration commands and daemons", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Kernel routines", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Other manual pages", icon, Unity.CategoryRenderer.HORIZONTAL_TILE)
                ]
	lens.props.filters = []

	lens.add_local_scope(ManPagesScope())
	lens.export()

	GObject.MainLoop().run()
